
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layPlugin.h"
#include "edtService.h"
#include "edtMainService.h"

namespace edt
{

class PluginDeclaration
  : public lay::PluginDeclaration
{
public:
  PluginDeclaration (const std::string &title, db::ShapeIterator::flags_type flags)
    : m_title (title), m_flags (flags)
  {
    //  .. nothing yet ..
  }

  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    //  .. the options are declared by the single main service but understood by the specific plugins ..
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const
  {
    return 0;
  }

  virtual void get_menu_entries (std::vector<lay::MenuEntry> &menu_entries) const
  {
    //  .. nothing yet ..
  }

  virtual lay::Plugin *create_plugin (db::Manager *manager, lay::MainWindow *, lay::LayoutView *view) const
  {
    if (m_flags == db::ShapeIterator::Nothing) {
      return new edt::Service (manager, view);
    } else {
      return new edt::Service (manager, view, m_flags);
    }
  }

  virtual bool implements_editable (std::string &title) const
  {
    title = m_title;
    return true;
  }
  
  virtual bool implements_mouse_mode (std::string &title) const
  {
    return false;
  }

private:
  std::string m_title;
  db::ShapeIterator::flags_type m_flags;
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl1 (new edt::PluginDeclaration (tl::translate ("Polygons"), db::ShapeIterator::Polygons), 4000, "edt::Service(Polygons)");
static tl::Registrar<lay::PluginDeclaration>::Class config_decl2 (new edt::PluginDeclaration (tl::translate ("Boxes"), db::ShapeIterator::Boxes), 4001, "edt::Service(Boxes)");
static tl::Registrar<lay::PluginDeclaration>::Class config_decl3 (new edt::PluginDeclaration (tl::translate ("Texts"), db::ShapeIterator::Texts), 4002, "edt::Service(Texts)");
static tl::Registrar<lay::PluginDeclaration>::Class config_decl4 (new edt::PluginDeclaration (tl::translate ("Paths"), db::ShapeIterator::Paths), 4003, "edt::Service(Paths)");
static tl::Registrar<lay::PluginDeclaration>::Class config_decl5 (new edt::PluginDeclaration (tl::translate ("Instances"), db::ShapeIterator::Nothing), 4010, "edt::Service(CellInstances)");

class MainPluginDeclaration
  : public lay::PluginDeclaration
{
public:
  MainPluginDeclaration (const std::string &title)
    : m_title (title)
  {
    //  .. nothing yet ..
  }

  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    //  .. nothing yet ..
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const
  {
    return 0;
  }

  virtual void get_menu_entries (std::vector<lay::MenuEntry> &menu_entries) const
  {
    lay::PluginDeclaration::get_menu_entries (menu_entries);
    menu_entries.push_back (lay::MenuEntry ("edt::hier_group", "zoom_menu.end"));
    menu_entries.push_back (lay::MenuEntry ("edt::descend", "descend", "zoom_menu.end", tl::translate ("Descend") + "(Ctrl+D)"));
    menu_entries.push_back (lay::MenuEntry ("edt::ascend", "ascend", "zoom_menu.end", tl::translate ("Ascend") + "(Ctrl+A)"));
  }

  virtual lay::Plugin *create_plugin (db::Manager *manager, lay::MainWindow *, lay::LayoutView *view) const
  {
    return new edt::MainService (manager, view);
  }

  virtual bool implements_editable (std::string &title) const
  {
    return false;
  }
  
  virtual bool implements_mouse_mode (std::string &title) const
  {
    return false;
  }

private:
  std::string m_title;
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl20 (new edt::MainPluginDeclaration (tl::translate ("Instances and shapes")), 4020, "edt::MainService");


}

